///////////////////////////////////////////////////////////////////////////
// $Id: managementWidget.cpp,v 1.5 1997/04/01 10:51:49 root Exp $	
// File  : managementWidget.cpp
// Author: Damyan Pepper
//
// See managementWidget.h for more information

#include "../config.h"
// kpackage.headers
#include "kpackage.h"
#include "managementWidget.moc"
#include "pkgInterface.h"
#include "pkguninstallDialog.h"

// constructor -- initialise variables
managementWidget::managementWidget(QWidget *parent, const char *name)
  : QFrame(parent,name)
{
  package=NULL;
  installedPackages=NULL;
  setupWidgets();
}

managementWidget::~managementWidget()
{
  if(installedPackages)
    delete installedPackages;
}

void managementWidget::resizeEvent(QResizeEvent *re)
{
  arrangeWidgets();
  re = re;
}


void managementWidget::setupWidgets()
{
  // Create the widgets
  treeList = new MyTreeList(this);
  packageDisplay = new packageDisplayWidget(this);
  uninstallButton = new QPushButton(klocale->translate("Uninstall"),this);
  uninstallButton->setEnabled(FALSE);

  // Connect the widgets
  connect(uninstallButton,SIGNAL(clicked()),
	  SLOT(uninstallClicked()));

  connect(treeList, SIGNAL(highlighted(int)),
	  SLOT(packageHighlighted(int)));

  connect(this, SIGNAL(changePackage(packageInfo *)),
	  packageDisplay, SLOT(changePackage(packageInfo *)));

  

  // Create the layout managers
  whole = new QBoxLayout(this, QBoxLayout::LeftToRight);
  rightpanel = new QBoxLayout(QBoxLayout::TopToBottom);
  buttons = new QBoxLayout(QBoxLayout::LeftToRight);

  // Setup the `whole' layout
  whole->addWidget(treeList);
  whole->addSpacing(2);
  whole->addLayout(rightpanel);

  // Setup the `right panel' layout
  rightpanel->addWidget(packageDisplay,10);
  rightpanel->addSpacing(5);
  rightpanel->addLayout(buttons,0);

  // Setup the `buttons' layout
  buttons->addStrut(20);
  buttons->addWidget(uninstallButton,1);
  buttons->addStretch(1);

  // Activate the layout managers
  whole->activate();
}

void managementWidget::arrangeWidgets()
{
  // this is done automatically by the layout managers
}

// Collect data from package.
void managementWidget::collectData(bool refresh)
{
  int i;

  if (!refresh && installedPackages)
    return; // if refresh not required already initialised

  QApplication::setOverrideCursor( waitCursor );

  // Delete old list if necessary
  if(installedPackages) {
    installedPackages->setAutoDelete(true);
    delete installedPackages;
  }

  installedPackages = new LpackageInfo;

  // List installed packages
  for (i = 0; i < kpinterfaceN; i++)  {
    kpinterface[i]->listInstalled(installedPackages);
  }

  // Rebuild the list tree
  rebuildListTree();

  QApplication::restoreOverrideCursor();
}

// Rebuild the list tree
void managementWidget::rebuildListTree()
{
  packageInfo *i;
  treeList->clear();		// empty the list tree
  treeList->hide();		// hide it

  // place all the packages found
  for(i=installedPackages->first(); i!=0; i=installedPackages->next())
    {
      i->place(treeList);
    }
  treeList->show();		// show the list tree 
  kpkg->kp->setStatus(klocale->translate("Management Mode"));	// set the status
  kpkg->kp->setPercent(100);	// set the progress
}

// A package has been highlighted in the list tree
void managementWidget::packageHighlighted(int index)
{
  int pt;
  const char *selected = treeList->itemAt(index)->getText();

  if (treeList->itemAt(index)->hasChild()) {
    emit changePackage(NULL);
    return;
  }

  pt = strrchr(selected,'\t') - selected;
  if (pt < 0) {
    pt = strlen(selected);
  }

  // Disable the uninstall button
  uninstallButton->setEnabled(FALSE);

  // Find which package has been selected
  packageInfo *i=NULL;
  for(i=installedPackages->first(); i!=0; i=installedPackages->next())
    {
      if(strncmp(i->getProperty("name")->data(), selected, pt) == 0)
	{
	  // Disable the tree list while we do this
	  treeList->setEnabled(FALSE);
	  // Tell everything that is interested to change packages
	  emit changePackage(i);

	  // Keep copy of the package
	  package = i;

	  // Re-enable the treeList and uninstall button
	  treeList->setEnabled(TRUE);
	  uninstallButton->setEnabled(TRUE);
	  break;
	}
    }

  // A non-package has been highlighted, so make sure that everything
  // knows this!
  if(!i)
    emit changePackage(NULL);

  // Update the status bar
  kpkg->kp->setStatus(klocale->translate("Management Mode"));
  kpkg->kp->setPercent(100);
}

// Uninstall has been clicked
void managementWidget::uninstallClicked()
{
  pkguninstallDialog *ui;
  int result;

  if (package)			// check that there is a package to uninstall
    {
      ui = new pkguninstallDialog(package,
	    package->interface->inituninstallOptions()); // create the uninstall dialog
      result = ui->exec();
      delete ui;

      if(result == QDialog::Accepted) // execute it
	{			// it was accepted, so the package has been
				// uninstalled
	  collectData(TRUE);	// collect data again
	  emit changePackage(NULL); // change package to no package
	  package = NULL;
	  uninstallButton->setEnabled(FALSE); // disable uninstall button
	}
    }
  kpkg->kp->setStatus(klocale->translate("Management Mode"));	// update the status bar
  kpkg->kp->setPercent(100);
}

int managementWidget::search(char *str, bool subStr, bool wrap)
{
  int n;
  KTreeViewItem  *item = treeList->itemAt(0);

  citem = treeList->getCurrentItem();

 do {
    n = searchChild(item,str,subStr,wrap);

    if (n != 0)
      return n;
      
  } while ((item = item->getSibling()));

 return n;
	   
}

int managementWidget::searchChild(KTreeViewItem *it,char *str, bool subStr, bool wrap)
{
  int r;


  if(it->hasChild()) {
    if (citem == it) {
      if (wrap) {
	return -1;
      } else {
	citem = 0;
      }
    }
    KTreeViewItem *childItem = it->getChild();
    while(childItem) {
      r = searchChild(childItem,str,subStr,wrap);
      if (r == 1)
	return 1;
      childItem = childItem->getSibling();
    }
  } else {
    if (citem == it) {
      if (wrap) {
	return -1;
      } else {
	citem = 0;
	return 0;
      }
    }
    if (citem && !wrap) {
      return 0;
    } else {
      QString s = it->getText().data();
      int n = s.find('\t',0,FALSE);
      if (n >= 0)
	s.truncate(n);
      if (subStr ? s.contains(str,FALSE) :s == str) {
	QStack<KTreeViewItem> stk;
	KTreeViewItem *i;

	i = it;
	while ((i = i->getParent())) {
	  stk.push(i);
	}
	while ((i = stk.pop())) {
	  treeList->expandItem(treeList->itemRow(i));
	}
	int c = treeList->itemRow(it);
	treeList->setCurrentItem(c);
	if (!treeList->rowIsVisible(c)) {
	  treeList->setTopCell(c - 1);
	}
	return 1;
      }
    }
  }
  return 0;
}
