////////////////////////////////////////////////////////////////////////////////
// File  : packageDisplay.cpp
// Author: Damyan Pepper
//
///////////////////////////////////////////////////////////////////////////////

#include "../config.h"
// Standard headers
#include <stdio.h>

// Qt headers

#include <qapp.h>
#include <qfileinf.h> 

// kpackage.headers
#include <kiconloader.h> 
#include "kpackage.h"
#include "packageDisplay.h"
#include "packageDisplay.moc"
#include "utils.h"
#include <kfm.h>
#include <kfmclient_ipc.h>

extern KIconLoader  *globalKIL; 

// constructor
packageDisplayWidget::packageDisplayWidget(QWidget *parent,
					   const char *name)
  : QFrame(parent,name)
{
  // Initially we're not dealing with any package
  package=NULL;

  // Set up the widgets
  setupWidgets();

  // Load the pixmaps
  tick = new QPixmap();
  *tick = globalKIL->loadIcon("ptick.xpm"); 

  cross = new QPixmap();
  *cross = globalKIL->loadIcon("cross.xpm"); 

  question = new QPixmap();
  *question = globalKIL->loadIcon("question.xpm");

  blank = new QPixmap();

}

packageDisplayWidget::~packageDisplayWidget()
{
  delete tick;
  delete cross;
  delete question;
}

void packageDisplayWidget::resizeEvent(QResizeEvent *re)
{
  arrangeWidgets();
  re = re;
}

void packageDisplayWidget::setupWidgets()
{
  tabbar = new QTabBar(this);
  QTab *proptab, *fltab;

  proptab = new QTab;
  fltab = new QTab;

  proptab->label = klocale->translate("Properties");
  proptab->enabled = FALSE;
  fltab->label = klocale->translate("File List");
  fltab->enabled = FALSE;

  packageProperties = new packagePropertiesWidget(this);
  fileList = new KTreeView(this);
  fileList->hide();
  connect(fileList, SIGNAL(selected(int)),
            this, SLOT( openBinding(int)) );
  tabbar->addTab(proptab);
  tabbar->addTab(fltab);

  connect(tabbar,SIGNAL(selected(int)), SLOT(tabSelected(int)));
}

void packageDisplayWidget::arrangeWidgets()
{
  const int correction = 7;
  int th = tabbar->height();
  int h = height() - tabbar->height() + correction;

  tabbar->move(0,0);
  tabbar->resize(width(), th);
  packageProperties->move(0,th-correction);
  packageProperties->resize(width(),h);
  fileList->move(0,th-correction);
  fileList->resize(width(),h);
}

void packageDisplayWidget::tabSelected(int tab)
{
  if(tab==0)
    {
      packageProperties->show();
      fileList->hide();
    }
  else
    {
      packageProperties->hide();
      fileList->show();
    }
}

// Change packages
void packageDisplayWidget::changePackage(packageInfo *p)
{
  int tfl = fileList->isVisible();
  // Disable the tabs
  tabbar->setTabEnabled(0,FALSE);
  tabbar->setTabEnabled(1,FALSE);
  // Delete the old package we were looking at
  if(package)
    delete package;

  char *filename;
  char mode;

  if(!p)			// change to no package
    {
      package = p;
      packageProperties->changePackage(NULL);
      return;
    }

  if(p->getFilename().data())	// get the filename, if there is one
    {
      mode = 'u';
      filename = p->getFilename().data();
    }
  else				// otherwise the filename is `name' from package
    {
      mode = 'i';
      filename = p->getProperty("name")->data();
    }

  // Grab information about the package 
  package = p->interface->getPackageInfo(mode,filename);

  if(package)			// there actually was one
    {
      package->setFilename(p->getFilename().data());
      packageProperties->changePackage(package);
      tabbar->setTabEnabled(0,TRUE);
      updateFileList();
      tabbar->setTabEnabled(1,TRUE);
    }
  else
    {
      packageProperties->changePackage(NULL);
      fileList->clear();
    }
  tabSelected(tfl);          
}

// update the file list
void packageDisplayWidget::updateFileList()
{
  // Get a list of files in the package
  QList<char> *files = 0;
  QList<char> *errorfiles = 0;

  // set the status
  kpkg->kp->setStatus(klocale->translate("Updating File List"));

  // clear the file list
  fileList->collapseItem(0);
  fileList->clear();

   // if there is no package then don't bother doing anything
  if(!package)
    return;

  // Check if the package is installed
  int installed;
  if(package->getFilename().isNull())
    installed=1;
  else
    installed=0;

  files = package->interface->getFileList(package);

  if(!files)
    return;

  // Get a list of files that failed verification
  if(installed) {
    errorfiles = package->interface->verify(package, files);
  } else
    errorfiles=NULL;
  
  kpkg->kp->setStatus(klocale->translate("Updating File List"));
  
  // Need a list iterator
  QListIterator<char> it(*files);

  int  verifyfailed=0;
  char *s;
  uint c=0, p=0;
  uint step = (it.count() / 100) + 1;

  QString ftmp;
  ftmp.setNum(it.count());
  ftmp += klocale->translate(" files");

  // a workaround for the tree widget being slow for long lists

  fileList->setExpandLevel(0);
  fileList->insertItem(ftmp.data(),*blank);

  // Go through all the files
  for(; (s = it.current()); ++it)
    {
      // Update the status progress
      c++;
      if(c > step)
	{
	  c=0;
	  p++;
	  kpkg->kp->setPercent(p);
	}

      int error=0;
      char *t;
      if(installed&&errorfiles)	// see if file failed verification,
	// if verification occured
	for(t=errorfiles->first(); t!=0; t=errorfiles->next())
	  {
	    if(strcmp(s,t)==0)
	      {
		error = 1;
		verifyfailed=1;
	      }
	  }

      // insert file in tree list with correct pixmap
      if(installed && errorfiles) {
	if(error) {
	   fileList->appendChildItem(s,*cross,0);
	} else {
	  fileList->appendChildItem(s,*tick,0);
	}
      } else
	fileList->appendChildItem(s,*question,0);
    }
  
  if(files) {
    delete files;
  }
  if(errorfiles) {
    delete errorfiles;
  }

  fileList->setExpandLevel(5);
}

void packageDisplayWidget::openBinding(int index)
  {
    QString tmp= "file:";
    KFM *kfm= new KFM();

    QFileInfo *fileInfo =
      new QFileInfo(fileList->itemAt(index)->getText());

    if (fileInfo->isDir())
      {
	tmp.append(fileInfo->filePath());
	kfm->openURL(tmp.data());
      }
    else
      {
	tmp.append(fileList->itemAt(index)->getText());
	kfm->exec(tmp.data(),0L);
      };
    delete kfm;
  };
